package bootstrapP;

import java.io.*;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**An implementation of a backupper that works on a single file.
 * @author Marco Cornolti
 *
 */
public class FileBackupper implements BackupperInterface {

	private File bkFile=null;
	
	
	/**The file backupper constructor.
	 * @param backupFilePath the file path to backup the data to.
	 */
	public FileBackupper(String backupFilePath){
		bkFile = new File(backupFilePath);
	}
	
	/**Get the file name of the backupper.
	 * @return the backupper file name.
	 */
	public String getFilename(){
		return  bkFile.getAbsolutePath();
	}
	
	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#backupWholeData(java.util.List, java.util.List)
	 */
	public synchronized void backupWholeData(List<RegisteredUser> rUsers, List<ManagedMessageServer> mMss) throws IOException{
		bkFile.createNewFile();
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(bkFile));
		
		Iterator<RegisteredUser> usersIt = rUsers.iterator();
		while (usersIt.hasNext()){
			oos.writeObject(usersIt.next());
		}
		
		Iterator<ManagedMessageServer> mssIt = mMss.iterator();
		while (mssIt.hasNext()){
			oos.writeObject(mssIt.next());
		}
		oos.flush();
		oos.close();
	}

	/**Backups or removes a single data from the file.
	 * @param obj the object to backup.
	 * @param remove true if the object must be removed from the backup file, false if the object must be saved.
	 * @throws IOException if an Input/Output problem occurred.
	 */
	public synchronized void backupSingleData(Object obj, boolean remove) throws IOException {
		List<RegisteredUser> users = new Vector<RegisteredUser>();
		List<ManagedMessageServer> mss = new Vector<ManagedMessageServer>();
		try{
			restoreData(users, mss);
		} catch (Exception e){
			throw new IOException();
		}
		if (obj instanceof RegisteredUser){
			RegisteredUser usr=(RegisteredUser)obj;
			if (remove)
				users.remove(usr);
			else
				users.add(usr);
		}
		if (obj instanceof ManagedMessageServer){
			ManagedMessageServer mms=(ManagedMessageServer) obj;
			if (remove)
				mss.remove(mms);
			else
				mss.add(mms);
		}
		backupWholeData(users, mss);

	}
	
	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#backupMs(bootstrapP.ManagedMessageServer)
	 */
	public synchronized void backupMs(ManagedMessageServer ms) throws IOException{
		backupSingleData(ms, false);
	}
	
	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#backupUser(bootstrapP.RegisteredUser)
	 */
	public synchronized void backupUser(RegisteredUser reg) throws IOException {
		backupSingleData(reg, false);
	}

	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#removeBackuppedMs(bootstrapP.ManagedMessageServer)
	 */
	public synchronized void removeBackuppedMs(ManagedMessageServer ms) throws IOException {
		backupSingleData(ms, true);
	}

	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#removeBackuppedUser(bootstrapP.RegisteredUser)
	 */
	public synchronized void removeBackuppedUser(RegisteredUser reg) throws IOException {
		backupSingleData(reg, true);
	}

	/* (non-Javadoc)
	 * @see bootstrapP.BackupperInterface#restoreData(java.util.List, java.util.List)
	 */
	public synchronized int restoreData(List<RegisteredUser> restoredUsers, List<ManagedMessageServer> restoredMss) throws IOException, ClassNotFoundException, InvalidClassException, StreamCorruptedException, OptionalDataException, IllegalArgumentException{
		bkFile.createNewFile();
		if (restoredUsers==null || restoredMss==null || !restoredMss.isEmpty() || !restoredUsers.isEmpty()) throw new IllegalArgumentException();
		
		int count=0;
		FileInputStream fis = new FileInputStream(bkFile);
		ObjectInputStream ois;
		try{
			ois = new ObjectInputStream(fis);
		} catch (EOFException e){
			return 0;
		}
		while (fis.available()>0){
			Object readObj = ois.readObject();
			count++;
			if (readObj instanceof RegisteredUser) restoredUsers.add((RegisteredUser) readObj);
			else if (readObj instanceof ManagedMessageServer) restoredMss.add((ManagedMessageServer) readObj);
			else throw new ClassCastException();
		}
		return count;
	}
}
