

//Generated by StarUML(tm) Java Add-In

//@ Project : Untitled
//@ File Name : HumanListener.java
//@ Date : 18/04/2007
//@ Author : 





package bootstrapP;

import java.io.IOException;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Vector;

import common.*;
import common.interfaces.exceptions.*;


/**The thread that listen for requests by the administrator and performs the correspondent actions.
 * @author Marco Cornolti
 *
 */
public class HumanListener extends Thread{
	public static HumanInterface currentInt = new TextualInterfaceIt();
	public void run() {
		currentInt.initialize();
		while (true){
			Command com = currentInt.getCommand();
			if (com.isAddMS()){
				executeAddMS(com);
			}
			else if (com.isDeleteUser()){
				executeDeleteUser(com);
			}
			else if (com.isMoveUser()){
				executeMoveUser(com);
			}
			else if (com.isRemoveMS()){
				executeRemoveMS(com);
			}
			else if (com.isShowAll()){
				executeShowAll();
			}
			else if (com.isShowGlobalStats()){
				executeShowGlobalStats();
			}
			else if (com.isBlockAssignation()){
				executeBlockAssignation(com,true);
			}
			else if (com.isUnblockAssignation()){
				executeBlockAssignation(com,false);
			}
			else if (com.isShutdown()){
				executeShutdown(com);
			}
			else if (com.isTestPresence()){
				executeTestPresence(com);
			}
			else if (com.isUsersList()){
				executeUserList(com);
			}
			else if (com.isBackup()){
				executeBackup(com);
			}
		}
	}


	/**Performs the "add message server" command.
	 * @param com the command given by the administrator, containing the message server data.
	 */
	private void executeAddMS(Command com){
		try{
			MessageServersDatabase.addMessageServer(InetAddress.getByName(com.getHostname()), com.getRMIPort(), com.getUDPPort(), com.getPassword());
		} catch (IOException e){
			currentInt.showCouldNotContactMS(com.getHostname(), com.getUDPPort(), com.getRMIPort());
		} catch (IllegalArgumentException e){
			currentInt.showIllegalArgument(com.getHostname(), com.getUDPPort(), com.getRMIPort());
		}
	}

	/**Performs the "delete user" command.
	 * @param com the command given by the administrator, containing the user data.
	 */
	private void executeDeleteUser(Command com){
		RegisteredUser reg = UsersDatabase.getUser(com.getNick());
		if (reg==null) {
			currentInt.showUserNotExists(com.getNick());
			return;
		}
		ManagedMessageServer managerMS = reg.getMS();
		int managerMSi = MessageServersDatabase.getMS(managerMS);
		try {
			managerMS.safeInt.deleteUserAdmin(new User(com.getNick(),null,0,false,false), managerMS.getPw());
			UsersDatabase.deleteUser(com.getNick());
			currentInt.showUserDeleted(com.getNick());
		} catch (WrongPasswordException e) {
			currentInt.showWrongMSPassword(managerMS.getIP().getCanonicalHostName(), managerMS.getUDPPort(), managerMS.getRMIPort());
			currentInt.showUserNotDeleted(com.getNick());
		} catch (BlockedException e) {
			currentInt.showMSBlocked(managerMSi);
			currentInt.showUserNotDeleted(com.getNick());
		} catch (UserNotManagedException e) {
			currentInt.showUserNotManaged(managerMSi, com.getNick());
			currentInt.showUserNotDeleted(com.getNick());
		} catch (RemoteException e){
			currentInt.showCouldNotContactMS(managerMS.getIP().getCanonicalHostName(), managerMS.getUDPPort(), managerMS.getRMIPort());
			currentInt.showUserNotDeleted(com.getNick());
		}
	}

	/**Performs the "move user" command.
	 * @param com the command given by the administrator, containing the user nickname and the message server id the user must be moved to.
	 */
	private void executeMoveUser(Command com){
		RegisteredUser reg = UsersDatabase.getUser(com.getNick());
		if (reg==null) {
			currentInt.showUserNotExists(com.getNick());
			return;
		}
		ManagedMessageServer ms1 = reg.getMS(); 
		ManagedMessageServer ms2 = MessageServersDatabase.getMS(com.getId1());
		if (ms1.equals(ms2)){
			currentInt.showUserMoveSameDest(com.getNick());
			return;
		}
		int ms1i=MessageServersDatabase.getMS(ms1);
		try {
			ms1.safeInt.moveUser(ms1.getPw(), new User(com.getNick(),null,0,false,false), ms2);
			UsersDatabase.refreshMovedUser(reg, ms2);
			currentInt.showUserMoved(com.getNick());
		} catch (WrongPasswordException e) {
			currentInt.showWrongMSPassword(ms1i);
			currentInt.showUserNotMoved(com.getNick());
		} catch (UserNotManagedException e) {
			currentInt.showUserNotManaged(ms1i, com.getNick());
			currentInt.showUserNotMoved(com.getNick());
		} catch (AssignationNotAcceptedException e) {
			currentInt.showMSUserNotAccepted(ms1i, com.getNick());
			currentInt.showUserNotMoved(com.getNick());
		} catch (GenericErrorException e) {
			currentInt.showMSGenericError(ms1i);
			currentInt.showUserNotMoved(com.getNick());
		} catch (RemoteException e){
			currentInt.showCouldNotContactMS(ms1.getIP().getCanonicalHostName(), ms1.getUDPPort(), ms1.getRMIPort());
			currentInt.showUserNotMoved(com.getNick());
		}

	}

	/**Performs the "remove message server" command.
	 * @param com the command given by the administrator, containing the message server id.
	 */
	private void executeRemoveMS(Command com){
		if (MessageServersDatabase.removeMS(com.getId1()))
			currentInt.showMSRemoved(com.getId1());
		else
			currentInt.showMSNotRemoved(com.getId1());
	}

	/**Performs the "show all" command.
	 */
	private void executeShowAll() {
		currentInt.showStatistics(BootstrapCore.getUpTime(), UsersDatabase.getUsersNumber(), MessageServersDatabase.getMSNumber());		
	}

	/**
	 * Performs the "show global statistics" command.
	 */
	private void executeShowGlobalStats() {
		List<ManagedMessageServer> mss = MessageServersDatabase.getAllMS();

		long minFreeMem = 0;
		int minFreeMemI = -1;

		long maxFreeMem = 0;
		int maxFreeMemI = -1;

		long fwMsgs=0;

		int minManUsers = 0;
		int minManUsersI = -1;

		int maxManUsers = 0;
		int maxManUsersI = -1;

		long onlineUsers = 0;
		long queuedMsgs = 0;

		for (int i=0; i<mss.size(); i++){
			ManagedMessageServer ms = mss.get(i);
			Statistics statsI = null;
			try{
				try {
					statsI = ms.safeInt.getStatistics(ms.getPw());
				} catch (WrongPasswordException e) {
					currentInt.showWrongMSPassword(ms.getIP().getCanonicalHostName(), ms.getUDPPort(), ms.getRMIPort());
					continue;
				}
			} catch (RemoteException e){
				currentInt.showCouldNotContactMS(ms.getIP().getCanonicalHostName(), ms.getUDPPort(), ms.getRMIPort());
				continue;
			}
			currentInt.showSingleMSStats(i, ms.getIP().getCanonicalHostName(), ms.getRMIPort(), ms.getUDPPort(), ms.getLoad(), statsI.getCacheEntries(),statsI.getFreeMem(), statsI.getForwardedMessagesMinute(), statsI.getForwardedMessagesHour(),statsI.getForwardedMessages24Hours(),statsI.getForwardedMessages144Hours(), statsI.getManagedUsers(), statsI.getOnlineManagedUsers(),statsI.getQueuedMessageNumber());

			if (statsI.getFreeMem() < minFreeMem || i==0){
				minFreeMem = statsI.getFreeMem();
				minFreeMemI = i;
			}
			if (statsI.getFreeMem() > maxFreeMem || i==0){
				maxFreeMem = statsI.getFreeMem();
				maxFreeMemI = i;
			}

			fwMsgs += statsI.getForwardedMessages144Hours();

			if (statsI.getManagedUsers() < minManUsers || i==0){
				minManUsers = statsI.getManagedUsers();
				minManUsersI = i;
			}
			if (statsI.getManagedUsers() > maxManUsers || i==0){
				maxManUsers = statsI.getManagedUsers();
				maxManUsersI = i;
			}

			onlineUsers+=statsI.getOnlineManagedUsers();
			queuedMsgs+=statsI.getQueuedMessageNumber();
		}

		String minFreeMemCHN = null;
		String maxFreeMemCHN = null;
		String minManUsersCHN = null;
		String maxManUsersCHN = null;
		if (minFreeMemI!=-1) minFreeMemCHN = mss.get(minFreeMemI).getIP().getCanonicalHostName();
		if (maxFreeMemI!=-1) maxFreeMemCHN = mss.get(maxFreeMemI).getIP().getCanonicalHostName();
		if (minManUsersI!=-1) minManUsersCHN = mss.get(minManUsersI).getIP().getCanonicalHostName();
		if (maxManUsersI!=-1) maxManUsersCHN = mss.get(maxManUsersI).getIP().getCanonicalHostName();
		currentInt.showGlobalMSStats(
				mss.size(),
				minFreeMem, minFreeMemI, minFreeMemCHN,
				maxFreeMem, maxFreeMemI, maxFreeMemCHN,
				minManUsers, minManUsersI, minManUsersCHN,
				maxManUsers, maxManUsersI, maxManUsersCHN,
				fwMsgs, onlineUsers, queuedMsgs);
	}

	/**Performs the "show user list" command.
	 * @param com the command given by the administrator, containing the id of the message server whose managed users must be shown. 
	 */
	private void executeUserList(Command com) {
		ManagedMessageServer ms = MessageServersDatabase.getMS(com.getId1());
		if (ms==null) {
			currentInt.showWrongMSId(com.getId1());
			return;
		}

		Vector<User> users=null;
		try{
			users = ms.safeInt.getUsersList(ms.getPw());
		} catch (WrongPasswordException e) {
			currentInt.showWrongMSPassword(com.getId1());
			return;
		} catch (RemoteException e){
			currentInt.showMSCommunicationProblem(com.getId1());
			return;
		}
		String userStr = "";
		for (int i=0;i<users.size();i++){
			userStr+=users.get(i).getNick()+"\n";
		}
		currentInt.showUserList(com.getId1(), userStr);
	}


	/**Performs the "test presence" command.
	 * @param com the command given by the administrator, containing the id of the message server whose presence must be checked.
	 */
	private void executeTestPresence(Command com) {
		ManagedMessageServer ms = MessageServersDatabase.getMS(com.getId1());
		if (ms==null) {
			currentInt.showWrongMSId(com.getId1());
			return;
		}
		try{
			ms.safeInt.testPresence();
			currentInt.showTestPresenceDone(com.getId1());
		} catch (RemoteException e){
			currentInt.showTestPresenceFailed(com.getId1());
		}
	}


	/**Performs the "shutdown" command.
	 * @param com the command given by the administrator, containing the id of the message server that must be shut down. 
	 */
	private void executeShutdown(Command com) {
		ManagedMessageServer ms = MessageServersDatabase.getMS(com.getId1());
		if (ms==null) {
			currentInt.showWrongMSId(com.getId1());
			return;
		}
		try{
			if (!ms.safeInt.testPassword(ms.getPw())){
				currentInt.showWrongMSPassword(com.getId1());
				return;
			}
		} catch (RemoteException e){
			currentInt.showMSCommunicationProblem(com.getId1());
			return;
		}
		try{
			ms.safeInt.shutdown(ms.getPw());
		} catch (WrongPasswordException e){
			currentInt.showWrongMSPassword(com.getId1());
			return;
		} catch (BlockedException e){
			currentInt.showCouldNotShutdownUnblockedMS(com.getId1());
			return;
		}catch (UsersStillPresentException e){
			currentInt.showMSShutdownFailedUsersPresent(com.getId1());
			return;
		} catch (RemoteException e){
			currentInt.showMSShutdownDone(com.getId1());
			return;
		}
	}


	/**Performs the "block assignation" and "unblock assignation" commands.
	 * @param com the command given by the administrator, containing the id of the message server whose assignations must be blocked.
	 * @param block	true if the assignation must be blocked, false if the assignation must be unblocked.
	 */
	private void executeBlockAssignation(Command com, boolean block) {
		ManagedMessageServer ms = MessageServersDatabase.getMS(com.getId1());
		if (ms==null) {
			currentInt.showWrongMSId(com.getId1());
			return;
		}
		try {
			ms.safeInt.blockAssignation(ms.getPw(), block);
			if (block) currentInt.showMSBlocked(com.getId1());
			else currentInt.showMSUnblocked(com.getId1());
			return;
		} catch (WrongPasswordException e){
			currentInt.showWrongMSPassword(com.getId1());
			return;
		} catch (RemoteException e){
			currentInt.showMSCommunicationProblem(com.getId1());
			return;
		}
	}

	/**Performs the "backup" command.
	 * @param com the command given by the administrator, containing the file name for the backup.
	 */
	private void executeBackup(Command com) {
		String filename=com.getFilename();
		BackupperInterface fb;

		if (filename !=null && !filename.equals("")){
			fb = new FileBackupper(filename);
			currentInt.showBackupFileStart(filename);
		}
		else{
			fb=BootstrapCore.mainBackupper;
			currentInt.showBackupStart();}
		try{
			fb.backupWholeData(UsersDatabase.getUsers(), MessageServersDatabase.getAllMS());
		} catch (Exception e){
			currentInt.showBackupFailed();
			return;
		}
		currentInt.showBackupDone();

	}
}
