

//Generated by StarUML(tm) Java Add-In

//@ Project : Untitled
//@ File Name : UsersDatabase.java
//@ Date : 16/04/2007
//@ Author : 





package bootstrapP;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

/**The database of users registered on the Bootstrap server.
 * @author Marco Cornolti
 */
public class UsersDatabase {
	static List<RegisteredUser> users = new Vector<RegisteredUser>();

	/**Returns a Registered User contained in the database from a given User.
	 * Takes log(n) where n is the number of registered users.
	 * @param user the user to search for (only the nickname is considered)
	 * @return null if the user is not contained in the database, the Registered User otherwise.
	 */
	public synchronized static RegisteredUser getUser(common.User user) {
		return getUser(user.getNick());
	}

	/**Returns a Registered User contained in the database from a given user nickname.
	 * Takes log(n) where n is the number of registered users.
	 * @param nick the nickname of the user to search for.
	 * @return null if the user is not contained in the database, the Registered User otherwise.
	 */
	public synchronized static RegisteredUser getUser(String nick) {
		int i = Collections.binarySearch(users, nick);
		if (i>=0) return users.get(i);
		else return null;
	}

	/**Registers a new user into the database.
	 * @param pw the user password.
	 * @param nick the user nickname.
	 * @param ms the message server the user is assignet to.
	 */
	public synchronized static void registerUser(String pw, String nick, ManagedMessageServer ms) {
		RegisteredUser reg = new RegisteredUser(nick,pw,ms);
		users.add(reg);
		Collections.sort(users);
		try{
			BootstrapCore.mainBackupper.backupUser(reg);
		} catch (Exception e){
			HumanListener.currentInt.showCouldNotBackupUser(nick);
		}

	}

	/**Deletes a user from the database.
	 * @param nick the nickname of the user to delete.
	 */
	public synchronized static void deleteUser(String nick) {
		RegisteredUser reg = new RegisteredUser(nick,null,null);
		int ind = Collections.binarySearch(users, nick);
		users.remove(ind);
		try{
			BootstrapCore.mainBackupper.removeBackuppedUser(reg);
		} catch (Exception e){
			HumanListener.currentInt.showCouldNotRemoveBackuppedUser(nick);
		}
	}

	/**Gets the total number of registered users.
	 * @return the total number of registered users.
	 */
	public synchronized static int getUsersNumber() {
		return users.size();
	}

	/**Restores the working users with those given by argument. Should only be used at boot time!
	 * @param restoredUsers the new users.
	 */
	public static synchronized void restoreData(List<RegisteredUser> restoredUsers) {
		users=restoredUsers;
		Collections.sort(users);
	}

	/**Returns the list of registered users. For backup purposals only. The returned list must not be modified.
	 * @return the list of registered users.
	 */
	public static synchronized List<RegisteredUser> getUsers(){
		return users;
	}
	
	public static synchronized void refreshMovedUser(RegisteredUser reg, ManagedMessageServer ms){
		reg.setMS(ms);
		try{
			BootstrapCore.mainBackupper.backupUser(reg);
		} catch (Exception e){
			HumanListener.currentInt.showCouldNotBackupUser(reg.getNick());
		}
	}
}
