//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : List.java
//  @ Date : 10/03/2007
//  @ Author : 
//
//



package common;

import java.io.Serializable;
import java.util.Vector;

import common.interfaces.exceptions.UserAlreadyInsertedException;


/**A representation of a user's contact list.
 * @author Marco Cornolti and Alessandro Franchini
 */
public class ContactList implements Serializable, Cloneable{
	
	private static final long serialVersionUID = 1L;
	Vector <User> vector;
    
    
	/**
	 * Creates a new empty contact list.
	 */
	public ContactList(){
    	vector = new Vector<User>();
    }
	/**
	 * @param user the user to search for.
	 * @return true only if the user given by argument is contained in the contact list.
	 */
	public boolean contains(User user) {
	    return vector.contains(user);
	}
    /** Adds a User nickname to the contact list. The User passed by argument is copied. The new User state is offline.
     * When the friend boolean is true, if the user is contained in the list but is not friend, it becomes friend.
     * @param user the User to add 
     * @param friend true if the user to add accepts messages from the user owning Contact List.
     * @throws UserAlreadyInsertedException if the user is already present and its friendness does not change.
     */
    public void addUser(User user, boolean friend) throws UserAlreadyInsertedException{
    	int idx = this.vector.indexOf(user);
        if (idx==-1){
        	vector.add(new User(user.getNick(),user.getIP(),user.getPort(),user.isOnline(),friend));
        	return;
        }
        User existing = this.vector.get(idx);
        if (existing.isFriend()) throw new UserAlreadyInsertedException();
        else if (!existing.isFriend() && !friend) throw new UserAlreadyInsertedException();
        else existing.setFriend(true);
        
        
    }
    
    /**Removes a User from the contact list.
     * @param user the User to delete
     * @return true if the user were removed, false otherwise (not present).
     */
    public boolean removeUser(User user){
        return this.vector.remove(user);
    }
    
    /**
     * @param i the user id.
     * @return a user of the index given as argument.
     */
    public User getUser(int i){
    	return vector.get(i);
    }
    
    /**
     * @param name the nickname.
     * @return the user whose nickname equals the one given by argument, or null if such a user does not exist.
     */
    public User getUser(String name){
		User tmp;
		for(int i=0;i<vector.size();i++){
			if((tmp = (User)(vector.get(i))).getNick().equals(name))
				return tmp;
			
		}
		return null;
	}
    
    /**Checks if the contact list accepts messages from a user given by argument.
     * @param user the user
     * @return true if and only if the contact list accepts messages from the user.
     */
    public boolean accepts(User user){
    	int idx = vector.indexOf(user);
    	if(idx==-1) return true;
    	if (!vector.get(idx).isFriend()) return false;
    	return true;
    }
    
    /**
     * @return the number of users contained into the contact list.
     */
    public int getSize(){
    	return vector.size();
    }
    
    
	/**Sets a contact status. If the contact is not present in the list, it is added.
	 * @param user the contact to change status of.
	 * @param friend true if the contact status must be set to friend, false if the contact must be blocked.
	 */
	public void setFriend(User user, boolean friend){
		int idx = this.vector.indexOf(user);
		try{
			addUser(user,friend);
		} catch (UserAlreadyInsertedException e){
			vector.elementAt(idx).setFriend(friend);
		}
	}

	/**
	 * Sets a contact status.
	 * @param user the contact to change status of.
	 * @param online true if the contatc is online, else false.
	 */
	public void setOnLine(User user, boolean online){
		int idx = this.vector.indexOf(user);
		vector.elementAt(idx).setOnline(online);
		if(online){
			vector.elementAt(idx).setIp(user.getIP());
			vector.elementAt(idx).setPort(user.getPort());
		}
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public synchronized ContactList clone(){
		ContactList clonedCL = new ContactList();
		  for (int i=0;i<this.vector.size();i++)
  	    	clonedCL.vector.add(this.vector.get(i).clone());
		  return clonedCL;
	}
}
