//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : MSForBSInterface.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//



package common.interfaces;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

import common.interfaces.exceptions.AssignationNotAcceptedException;
import common.interfaces.exceptions.BlockedException;
import common.interfaces.exceptions.GenericErrorException;
import common.interfaces.exceptions.UserNotManagedException;
import common.interfaces.exceptions.UsersStillPresentException;
import common.interfaces.exceptions.WrongPasswordException;


/** This interface represents the method that a Gossip Message Server must provide to let a Bootstrap work.
 * @author Marco Cornolti and Alessandro Franchini
 */
public interface MSForBSInterface extends Remote{
	/**Tests the presence of a Message Server
	 * @throws RemoteException if the server could not be reached.
	 */
	public abstract void testPresence() throws RemoteException;
	/** Deletes a user. This method is invoked by the global administrator.
	 * @param nick the user to delete.
	 * @param MSpw the Message Server administration password.
	 * @throws RemoteException if a communication problem with the server occurred.
	 * @throws WrongPasswordException if the password is not right.
	 * @throws BlockedException if the MS is currently blocked.
	 * @throws UserNotManagedException if the specified user is not managed by the Message server.
	 */
	public abstract void deleteUserAdmin(common.User nick, String MSpw)  throws RemoteException, WrongPasswordException, BlockedException, UserNotManagedException ;
	/**Puts the MS in the 'blocked' or 'unblocked' status, depending on the boolean <b>status</b> given by argument.
	 * If the MS is blocked, no new assignations are accepted.
	 * The MS must be put in bloked status when their users have to be transferred to another
	 * MS before shutting down. 
	 * @param MSPw the MS administration password.
	 * @param status true if the status must be set to 'blocked', false if the status must be set to 'unblocked'.
	 * @throws RemoteException if a communication problem occurs.
	 * @throws WrongPasswordException if the password is wrong.
	 */
	public abstract void blockAssignation(String MSPw, boolean status)  throws RemoteException, WrongPasswordException;
	/**Moves a user assignation from the message server to the specified one.
	 * @param MSPw the message server password.
	 * @param user the user to move.
	 * @param ms the message server to move the user to.
	 * @throws RemoteException if a communication problem occurs.
	 * @throws WrongPasswordException id the MS admin password is wrong.
	 * @throws UserNotManagedException if the user is not managed by this MS (or does not exists).
	 * @throws AssignationNotAcceptedException if the MS the user was wanted to be moved to refused the assignation (i.e. is blocked).
	 * @throws GenericErrorException if a communication problem with the second MS occurred.
	 */
	public abstract void moveUser(String MSPw, common.User user, common.MessageServer ms)  throws RemoteException, WrongPasswordException, UserNotManagedException, AssignationNotAcceptedException, GenericErrorException;
	/**Returns the statistics of the message server.
	 * @param globalpw the MS password
	 * @return an object containing the statistics of the message server.
	 * @throws RemoteException if a communcation problem occurs.
	 * @throws WrongPasswordException if the administration password is wrong.
	 */
	public abstract common.Statistics getStatistics(String globalpw) throws RemoteException, WrongPasswordException;
	/**Shuts down the MS server. Must be invoked when no users are managed by the MS 
	 * and the MS is blocked. 
	 * @param MSPw the Message Server password.
	 * @throws RemoteException if there was a communication error or the MS was correctly shutdown.
	 * @throws WrongPasswordException if the password is wrong.
	 * @throws UsersStillPresentException if the Message Server still manages some users.
	 * @throws BlockedException if the Message Server is not blocked.
	 */
	public abstract void shutdown(String MSPw) throws RemoteException, WrongPasswordException, UsersStillPresentException, BlockedException;
	/**Assigns a new user to the message server.
	 * @param MSPw the MS password.
	 * @param user the user to add
	 * @param userPw the user password
	 * @throws BlockedException if the server is blocked and does not accept new users.
	 * @throws WrongPasswordException if the Message Server Password is wrong. 
	 * @throws RemoteException if a communication problem occurs.
	 */
	public abstract void assign(String MSPw, common.User user, String userPw) throws RemoteException, BlockedException, WrongPasswordException;
	
	/**Returns the list of users managed by the Message Server. (Used only for debug purposals?)
	 * @param globalpw the MS password
	 * @return a vector containing the managed users or null if the password is wrong.
	 * @throws WrongPasswordException if the password is wrong.
	 * @throws RemoteException if a communication problem occurs.
	 */
	public abstract Vector<common.User> getUsersList(String globalpw) throws RemoteException, WrongPasswordException;
	
	/**Tests a message server administration password.
	 * @param pw the password to check
	 * @return true if the password is right, false otherwise. 
	 */
	public abstract boolean testPassword(String pw) throws RemoteException;

}
