

//Generated by StarUML(tm) Java Add-In

//@ Project : Untitled
//@ File Name : Cache.java
//@ Date : 19/03/2007
//@ Author : 





package messageServerP;

import java.rmi.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import common.*;
import common.interfaces.*;
import common.interfaces.exceptions.*;


/**
 *Contains the MessageServer to User correspondence cached by the message server.
 *   @author Marco Cornolti
 */
public class Cache {

	/** The Bootstrap Server interface*/
	public static BSForMSInterface BSInterf;

	/** The Bootstrap Server (must be set at startup with setBootstrap(Bootstrap bs) )*/
	private static Bootstrap bootstrap;


	/** The list of users not managed by this MS but cached.
	 * If a user is in the vector of index i, that means that the user is assigned to the
	 * message server messageServers.get(i) whose interface is MSInterfs.get(i)
	 */ 
	private static Vector<List<User>> nonManagedUsers = new Vector<List<User>>();

	/** The Message Servers interfaces. */
	private static Vector <SafeMSForMSInterface> safeMSInterfs  = new Vector<SafeMSForMSInterface>();

	/** The Message Servers. */
	private static Vector <MessageServer> messageServers = new Vector<MessageServer>();


	/**Sets the bootstrap to globally use and creates the interface.
	 * @param bs the bootstrap server
	 * @throws NotBoundException
	 * @throws RemoteException
	 */
	public static void setBootstrap(Bootstrap bs) throws NotBoundException, RemoteException{
		Registry bsReg = LocateRegistry.getRegistry(bs.getIP().getHostAddress(), bs.getRMIPort());
		Cache.BSInterf = (BSForMSInterface) bsReg.lookup("gossipBS");

		bootstrap=bs;
	}

	/**
	 * @return the currently used bootstrap server.
	 */
	public static Bootstrap getBS(){
		return bootstrap;
	}

	/**Obtains from the cache the Message Server interface used to communicate with a user.
	 * Takes O(M*logN) where N is the amount of cached User-MS corrispondence, and M is the amount of servers.
	 * @param user the user whose Message Server interface must be searched for.
	 * @return the user's Message Server interface, null if not found.
	 */
	public static synchronized SafeMSForMSInterface getCachedMessageServerInterface(User user) {
		for (int i=0;i<nonManagedUsers.size();i++){
			if (Collections.binarySearch(nonManagedUsers.get(i), user)>=0) return safeMSInterfs.get(i);
		}
		return null;
	}

	/**Obtains from the cache the Message Server a user is assigned to.
	 * @param user the user whose Message Server must be searched for.
	 * @return the user's Message Server, null if not found.
	 */
		private static synchronized MessageServer getCachedMessageServer(User user) {
		for (int i=0;i<nonManagedUsers.size();i++)
			if (Collections.binarySearch(nonManagedUsers.get(i), user)>=0) return messageServers.get(i);
		return null;
	}
	/**Forwards a message to another Message Server (recipient must not be local).
	 * Updates the cache if needed. 
	 * @param msg The message to forward.
	 * @throws GenericErrorException if a communication error occurred.
	 * @throws RemoteException if the communication with a message server failed.
	 * @throws UserNotExistsException if the Bootstrap server cannot tell the user MS.
	 * @throws SystemErrorException if a system error occurred (Bootstrap thinks the user is
	 * assigned to a message server that does not actually manage it).
	 * @throws NotAllowedException if the recipient does not accept messages from the sender.
	 */
	public static synchronized void forwardMsg(Message msg) throws GenericErrorException, RemoteException, UserNotExistsException, SystemErrorException, NotAllowedException {
		User recipient = msg.getTo();
		SafeMSForMSInterface destMSInt=getCachedMessageServerInterface(recipient);

		if (destMSInt==null)//unknown recipient MS
			if (!refreshCache(recipient)) throw new UserNotExistsException();
			else destMSInt=getCachedMessageServerInterface(recipient); 
				
		//known recipient MS
		try{
			destMSInt.forwardMessage(msg);
			//right recipient MS
			return;
		} catch (UserNotManagedException e){
			if (!refreshCache(recipient)) throw new SystemErrorException();
			destMSInt=getCachedMessageServerInterface(recipient);
			forwardMsg(msg);
			return;
		}
		
	}

	/**Checks if a remote user accepts a local user as a friend. 
	 * @param local the local user
	 * @param remote the remote user (must not be local!)
	 * @throws RemoteException if a communication problem with another server occurred. 
	 */
	public static synchronized void remoteIsFriendOf(ManagedUser local, User remote) throws GenericErrorException, UserNotExistsException, SystemErrorException, RemoteException {
		SafeMSForMSInterface destMSInt=getCachedMessageServerInterface(remote);

		if (destMSInt==null)//unknown remote user MS
			if (!refreshCache(remote)) throw new UserNotExistsException();
			else destMSInt=getCachedMessageServerInterface(remote);

		//known recipient MS
		try{
			destMSInt.inListOfUserContains(remote, local);
		} catch (UserNotManagedException e){
			if(!refreshCache(remote)) throw new SystemErrorException();
			remoteIsFriendOf(local, remote);
		}

		//right recipient MS
		return;
	}
	
	/**Retrives informations about a gossip servlet connection (IP address and port) in order to let anyone contact
	 * him directely.
	 * @param usr the user to get the contact of.
	 * @return the same user with the IP and port set.
	 * @throws RemoteException if a problem occurred while connecting to the bootstrap server.
	 * @throws UserNotExistsException if the user does not exist in the system.
	 * @throws SystemErrorException if a system error occurred (Bootstrap thinks the user is
	 * assigned to a message server that does not actually manage it).
	 */
	public static User getRemoteContact(User usr) throws RemoteException, UserNotExistsException, SystemErrorException, GenericErrorException{
		SafeMSForMSInterface msi = getCachedMessageServerInterface(usr);
		if (msi==null)  //user not in cache
			if (!refreshCache(usr)) throw new UserNotExistsException();
			else msi=getCachedMessageServerInterface(usr);
		
		try{//user in cache
			return msi.getManagedUserContact(usr);
		} catch (UserNotManagedException e){
			//user-MS cache corrispondence wrong
			if (!refreshCache(usr)) throw new SystemErrorException();
			return getRemoteContact(usr);
		}
		
	}

	/**Refreshes the cache for a user whose message server is not right (or is not cached),
	 * sending a query to the Bootstrap Server. 
	 * @param user the user whose cached message server is not right
	 * @return true if the message server has been updated, false if the bootstrap query returned the old message server (or the user does not exist).
	 * @throws RemoteException
	 * @throws GenericErrorException 
	 */
	private static synchronized boolean refreshCache(User user) throws RemoteException, GenericErrorException {
		MessageServer incorrectMS = getCachedMessageServer(user);
		MessageServer newMS;
		try{
			try{
				newMS = BSInterf.getMS(user);
			} catch (RemoteException e1){
				try{
					setBootstrap(bootstrap);
					newMS = BSInterf.getMS(user);
				} catch (RemoteException e){
					throw e;
				} catch (NotBoundException e){
					throw new RemoteException();
				} catch (UserNotExistsException e){
					throw e;
				}
			}
		} catch (UserNotExistsException e){
			removeFromCache(user);
			return false;
		}
		
		if (newMS.equals(incorrectMS)) return false;
		removeFromCache(user);
		addToCache(user,newMS);
		return true;

	}


	/**Moves a given user (including its message queue and contact lists)
	 * from the local message to another. does not remove the user from the local ms.
	 * NOTE: the managed user passed as argument is blocked and cannot receive messages
	 * until it gets unblocked.
	 * @param user the user to move
	 * @param ms the message server to move the user to
	 * @throws GenericErrorException if the message server could not be contacted.
	 * @throws RemoteException if a communication problem occurred.
	 * @throws AssignationNotAcceptedException if the server refused the user assignation.
	 */
	public static void sendUser(ManagedUser user, MessageServer ms) throws GenericErrorException, RemoteException,AssignationNotAcceptedException{
		SafeMSForMSInterface msint;
		synchronized (Cache.class){
			int idx = messageServers.indexOf(ms);
			if (idx==-1) {
				addEmptyMS(ms);
				idx = messageServers.size()-1;
			}
			msint = safeMSInterfs.get(idx);
		}
		msint.assignMovedUser(user);
	}



	/**Updates the cache adding a new corrispondence between a user and the message server the user is located in.
	 * The user must not be locally managed. 
	 * Takes O(NlogN) where N is the number of users already assigned to ms.
	 * @param user the user
	 * @param ms the Message Server
	 */
	private static synchronized void addToCache(common.User user, common.MessageServer ms) throws GenericErrorException{
		removeFromCache(user);

		int i;
		List<User> listToAddTo;
		
		if ((i = (messageServers.indexOf(ms))) != -1){
			listToAddTo = nonManagedUsers.get(i);
		}
		else{
			int j = addEmptyMS(ms);
			listToAddTo = nonManagedUsers.get(j);
		}
		listToAddTo.add(user);
		Collections.sort(listToAddTo);
	}
	
	/**Creates an empty message server with no users assigned to. If the message server already exists, returns its
	 * index.  
	 * @param msToAdd the message server to add
	 * @return the new (or old) message server index in the vectors messageServer, MSInterfs, nonManagedUsers.
	 * @throws GenericErrorException if the message server could not be reached.
	 */
	private static synchronized int addEmptyMS (MessageServer msToAdd) throws GenericErrorException{
		if (messageServers.contains(msToAdd)) return messageServers.indexOf(msToAdd);
		try{
			safeMSInterfs.add(new SafeMSForMSInterface (msToAdd.getIP(), msToAdd.getPort()));
		} catch (Exception e){
			throw new GenericErrorException();
		} 
		messageServers.add(msToAdd);
		nonManagedUsers.add(new Vector<User>());
		return messageServers.size()-1;
	}

	/**If exists, removes the correspondence between a user and the Message Server
	 * @param user the user to delete from cache
	 * @return the old Message Server the user were associated to
	 */
	public static synchronized MessageServer removeFromCache(common.User user) {
		for (int i=0;i<nonManagedUsers.size();i++){
			if (nonManagedUsers.get(i).remove(user)){
				MessageServer oldMS = messageServers.get(i);
				if (nonManagedUsers.get(i).isEmpty()){
					nonManagedUsers.remove(i);
					messageServers.remove(i);
					safeMSInterfs.remove(i);
				}
				return oldMS;
			}
		}
		return null;
	}


	/**Returns the number of user-MS corrispondences contained in the cache.
	 * @return the number of user-MS corrispondences contained in the cache.
	 */
	public static synchronized int getCacheEntries(){
		int entries=0;
		for (int i=0;i<nonManagedUsers.size();i++)
			entries+=nonManagedUsers.get(i).size();
		return entries;
	}
}
