

//Generated by StarUML(tm) Java Add-In

//@ Project : Untitled
//@ File Name : Communication.java
//@ Date : 19/03/2007
//@ Author : 





package messageServerP;

import java.io.IOException;
import java.io.Serializable;
import java.net.*;
import java.rmi.RemoteException;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import common.*;
import common.interfaces.exceptions.*;


/**
 * <p>The message server Communication class implements all the communications
 * interfaces offered by the message server with RMI. It also implements some methods
 * using other internet communication paradigms.<br>
 * This class is security critical because it contains all the methods accessible by
 * other hosts on the Internet</p>
 * 
 * <p>one line to give the program's name and an idea of what it does.
 * Copyright (C) 2005  Marco Cornolti (cornolti@cli.di.unipi.it)</p>
 *
 * <p>This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.</p>
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.</p>
 * 
 * @author Marco Cornolti
 * 
 */

public class Communication extends UnicastRemoteObject implements common.interfaces.MSForBSInterface, common.interfaces.MSForServletInterface, common.interfaces.MSForMSInterface, Serializable {

	/**
	 * Gossip Message Server version
	 */
	private static final long serialVersionUID = 1L;


	/**Constructor for the remote object.
	 * @throws RemoteException if the remote object could not be created.
	 */
	protected Communication() throws RemoteException {
		super();
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#testPresence()
	 */
	public void testPresence() {
		HumanListener.currentInt.showTestPresence();
		return;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#deleteUserAdmin(common.User, java.lang.String)
	 */
	public void deleteUserAdmin(common.User nick, String MSpw) throws WrongPasswordException, BlockedException, UserNotManagedException{
		HumanListener.currentInt.showAdminDeletionDone(nick.getNick());
		if (ManagedUsersDatabase.assignationBlocked()) throw new BlockedException();
		if (!MyselfMessageServer.myself.checkPassword(MSpw)) {
			HumanListener.currentInt.showAdminWrongPassword();
			throw new WrongPasswordException();
		}
		synchronized(ManagedUsersDatabase.class){ /**All the methods that affect the User's datas are synchronized with those method affecting the Users existance on the class ManagedUserDatabase*/
			if (ManagedUsersDatabase.deleteUser(nick)) {
				HumanListener.currentInt.showAdminDeletionRequest(nick.getNick());
				return;
			}
			else throw new UserNotManagedException();
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#blockAssignation(java.lang.String, boolean)
	 */
	public void blockAssignation(String MSPw, boolean status) throws WrongPasswordException {
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) throw new WrongPasswordException();
		if (status) HumanListener.currentInt.showAssignationBlocked();
		else HumanListener.currentInt.showAssignationUnblocked();
		ManagedUsersDatabase.blockAssignation(status);
		return;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#moveUser(java.lang.String, common.User, common.MessageServer)
	 */
	public void moveUser(String MSPw, common.User user, common.MessageServer ms) throws WrongPasswordException, UserNotManagedException, RemoteException, AssignationNotAcceptedException, GenericErrorException {
		HumanListener.currentInt.showMoveRequest(user.getNick(),ms.toString());
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) {
			HumanListener.currentInt.showAdminWrongPassword();
			throw new WrongPasswordException();
		}
		synchronized(ManagedUsersDatabase.class){
			ManagedUser manUser = ManagedUsersDatabase.getManaged(user);
			if (manUser==null) throw new UserNotManagedException();
			ManagedUsersDatabase.blockUser(user,true);

			try{
				Cache.sendUser(manUser, ms);
			} catch (GenericErrorException e){
				ManagedUsersDatabase.blockUser(user,false);
				throw e;
			}
			HumanListener.currentInt.showMoveDone(user.getNick(),ms.toString());
			ManagedUsersDatabase.deleteUser(user);
			return;
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#getStatistics(java.lang.String)
	 */
	public common.Statistics getStatistics(String MSPw) throws WrongPasswordException {
		HumanListener.currentInt.showStatsRequest();
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) {
			HumanListener.currentInt.showAdminWrongPassword();
			throw new WrongPasswordException();
		}
		return new Statistics(Cache.getCacheEntries(),ManagedUsersDatabase.getManagedUsers(),ManagedUsersDatabase.getOnlineManagedUsers(),ManagedUsersDatabase.getQueuedMessagesNum());
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#shutdown(java.lang.String)
	 */
	public void shutdown(String MSPw) throws UsersStillPresentException, WrongPasswordException, BlockedException {
		HumanListener.currentInt.showShutdownRequest();
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) throw new WrongPasswordException();
		if (!ManagedUsersDatabase.assignationBlocked()) throw new BlockedException();
		if (!HumanListener.executeShutdown()) 
			throw new UsersStillPresentException();
		return;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#assign(java.lang.String, common.User, java.lang.String)
	 */
	public void assign(String MSPw, common.User user, String password) throws BlockedException, WrongPasswordException {
		HumanListener.currentInt.showAssignationRequest(user.getNick());
		if (ManagedUsersDatabase.assignationBlocked()) throw new BlockedException();
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) {
			HumanListener.currentInt.showAdminWrongPassword();
			throw new WrongPasswordException();
		}
		ManagedUsersDatabase.createUser(user,password);
		HumanListener.currentInt.showAssignationDone(user.getNick());
		return;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#getUsersList(java.lang.String)
	 */
	public Vector<User> getUsersList(String MSPw) throws WrongPasswordException {
		HumanListener.currentInt.showUsersListRequest();
		if (!MyselfMessageServer.myself.checkPassword(MSPw)) {
			HumanListener.currentInt.showAdminWrongPassword();
			throw new WrongPasswordException();
		}
		HumanListener.currentInt.showUsersListDone();
		
		Vector<User> vect = new Vector<User>();
		User[] usrs = ManagedUsersDatabase.getManagedUsersArray();
		for (int i=0;i<usrs.length;i++){
			vect.add(usrs[i]);
		}
		return vect;
	}


	/* (non-Javadoc)
	 * @see common.interfaces.MSForBSInterface#testPassword(java.lang.String)
	 */
	public boolean testPassword(String pw) {
		return MyselfMessageServer.myself.checkPassword(pw);
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#addToFriends(common.User, java.lang.String, common.User)
	 */
	public void addToFriends(common.User user, String userPw, common.User userToAdd) throws UserNotManagedException, WrongPasswordException, UserBlockedException, UserAlreadyInsertedException {
		HumanListener.currentInt.showAddToFriendRequest(user.getNick(), userToAdd.getNick());
		synchronized(ManagedUsersDatabase.class){
			ManagedUser manUser = ManagedUsersDatabase.getManaged(user);
			if (manUser==null) {
				HumanListener.currentInt.showWrongUser(user.getNick());
				throw new UserNotManagedException();
			}
			if (!manUser.checkPassword(userPw)) {
				HumanListener.currentInt.showWrongUserPassword(user.getNick());
				throw new WrongPasswordException();}
			if (manUser.isBlocked()) {
				HumanListener.currentInt.showUserBlocked(user.getNick());
				throw new UserBlockedException();
			}
			try {
				manUser.addFriend(userToAdd);
				HumanListener.currentInt.showAddToFriendDone(user.getNick(),userToAdd.getNick());
			} catch (UserAlreadyInsertedException e){
				HumanListener.currentInt.showAddAlreadyInserted(user.getNick(),userToAdd.getNick());
				throw e;
			}
			return;
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#downloadMsgs(common.User, java.lang.String)
	 */
	public Vector<Message> downloadMsgs(common.User user, String pw) throws UserNotManagedException, UserBlockedException, WrongPasswordException{
		HumanListener.currentInt.showDownloadRequest(user.getNick());
		ManagedUser manUser = ManagedUsersDatabase.getManaged(user);
		if (manUser==null) {
			HumanListener.currentInt.showWrongUser(user.getNick());
			throw new UserNotManagedException();
		}
		if (!manUser.checkPassword(pw)) {
			HumanListener.currentInt.showWrongUserPassword(user.getNick());
			throw new WrongPasswordException();
		}
		if (manUser.isBlocked()) {
			HumanListener.currentInt.showUserBlocked(user.getNick());
			throw new UserBlockedException();
		}
		HumanListener.currentInt.showDownloadDone(user.getNick());
		return manUser.getQueue();
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#setState(common.User, java.lang.String, boolean)
	 */
	public void setState(common.User user, String pw, boolean state) throws UserNotManagedException, WrongPasswordException, GenericErrorException, UserBlockedException {
		ManagedUser local;
		HumanListener.currentInt.showSetStatusRequest(user.getNick(), state);
		synchronized(ManagedUsersDatabase.class){
			local=ManagedUsersDatabase.getManaged(user);
			if (local==null) {
				HumanListener.currentInt.showWrongUser(user.getNick());
				throw new UserNotManagedException();
			}
			if (!local.checkPassword(pw)){
				HumanListener.currentInt.showWrongUserPassword(user.getNick());
				throw new WrongPasswordException();
			}
			if (local.isBlocked()) throw new UserBlockedException();
			try {
				try{
//					TODO: controlla se funziona dietro la nat
					local.setIp(InetAddress.getByName(getClientHost()));
					local.setPort(user.getPort());
				} catch (UnknownHostException e2){
					throw new GenericErrorException();
				}
			} catch (ServerNotActiveException e) {
				throw new GenericErrorException();
			}
			local.setOnline(state);
			HumanListener.currentInt.showSetStatusDone(user.getNick(), state);
			return;
		}
	}


	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#forwardMessage(common.Message)
	 */
	public void forwardMessage(common.Message msgRef) throws NotAllowedException, UserNotManagedException, RemoteException, GenericErrorException, UserNotExistsException, SystemErrorException{
		common.Message msg = msgRef.clone();
		HumanListener.currentInt.showForwardRequest(msg.getFrom().getNick(), msg.getTo().getNick(),msg.getText().length());
		ManagedUser local;
		synchronized(ManagedUsersDatabase.class){
			Statistics.increaseForwardedMessages();
			if ((local = ManagedUsersDatabase.getManaged(msg.getTo()))!=null) { //local recipient
				if (local.isFriendOf(msg.getFrom())){
					local.addMsgToQueue(msg);
					HumanListener.currentInt.showForwardLocalDone(msg.getFrom().getNick(), msg.getTo().getNick(),msg.getText().length());
					return;}
				else {
					HumanListener.currentInt.showForwardNotAllowed(msg.getFrom().getNick(), msg.getTo().getNick(),msg.getText().length());
					throw new NotAllowedException();
				}
			}
			if (ManagedUsersDatabase.getManaged(msg.getFrom()) == null){
				HumanListener.currentInt.showWrongUser(msg.getFrom().getNick());
				throw new UserNotManagedException();
			}

			HumanListener.currentInt.showForwardCacheDone(msg.getFrom().getNick(), msg.getTo().getNick(),msg.getText().length());
			Cache.forwardMsg(msg);
			HumanListener.currentInt.showForwardDone(msg.getFrom().getNick(), msg.getTo().getNick(),msg.getText().length());
		}
	}


	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#getContactList(common.User, java.lang.String)
	 */
	public common.ContactList getContactList(common.User user, String pw) throws UserNotManagedException, WrongPasswordException, UserBlockedException{
		HumanListener.currentInt.showContactListRequest(user.getNick());
		ManagedUser local;
		local=ManagedUsersDatabase.getManaged(user);
		if (local==null) {
			HumanListener.currentInt.showWrongUser(user.getNick());
			throw new UserNotManagedException();
		}
		if (!local.checkPassword(pw)) {
			HumanListener.currentInt.showWrongUserPassword(user.getNick());
			throw new WrongPasswordException();
		}
		if (local.isBlocked()) throw new UserBlockedException();
		HumanListener.currentInt.showContactListDone(user.getNick());
		return local.getContactList();
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#changePassword(common.User, java.lang.String, java.lang.String)
	 */
	public void changePassword(common.User user, String oldPw, String newPw) throws UserNotManagedException, WrongPasswordException, UserBlockedException{
		HumanListener.currentInt.showChangePasswordRequest(user.getNick());
		ManagedUser local;
		synchronized(ManagedUsersDatabase.class){
			local=ManagedUsersDatabase.getManaged(user);
			if (local==null) {
				HumanListener.currentInt.showWrongUser(user.getNick());
				throw new UserNotManagedException();
			}
			if (!local.checkPassword(oldPw)) {
				HumanListener.currentInt.showWrongUserPassword(user.getNick());
				throw new WrongPasswordException();
			}
			if (local.isBlocked()) throw new UserBlockedException();
			else {
				local.changePassword(oldPw,newPw);
				HumanListener.currentInt.showChangePasswordDone(user.getNick());
				return;
			}
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#testPorts(int, java.net.InetAddress)
	 */
	public boolean testPorts(int port, InetAddress ip) {
		HumanListener.currentInt.showTestPortsRequest(port, ip.getHostAddress());
		try{
			new Socket(ip,port);
		}catch (Exception e){
			HumanListener.currentInt.showTestPortsFailed(port, ip.getHostAddress());
			return false;
		}
		HumanListener.currentInt.showTestPortsDone(port, ip.getHostAddress());
		return true;

	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForMSInterface#inListOfUserContains(common.User, common.User)
	 */
	public boolean inListOfUserContains(common.User listUser, common.User user) throws UserNotManagedException {
		HumanListener.currentInt.showInListOfUserContainsRequest(listUser.getNick(),user.getNick());
		ManagedUser man;
		if ((man = ManagedUsersDatabase.getManaged(listUser))==null) {
			HumanListener.currentInt.showWrongUser(listUser.getNick());
			throw new UserNotManagedException();
		}
		if(man.isFriendOf(user)) {
			HumanListener.currentInt.showInListOfUserContainsDone(listUser.getNick(),user.getNick(), true);
			return true;
		}
		HumanListener.currentInt.showInListOfUserContainsDone(listUser.getNick(),user.getNick(), false);
		return false;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForMSInterface#assignMovedUser(messageServerP.ManagedUser)
	 */
	public void assignMovedUser(ManagedUser user) throws AssignationNotAcceptedException{
		HumanListener.currentInt.showAssignMovedRequest(user.getNick());
		if (ManagedUsersDatabase.assignationBlocked()) {
			HumanListener.currentInt.showUserBlocked(user.getNick());
			throw new AssignationNotAcceptedException();
		}
		ManagedUser newUser = user.clone();
		newUser.block(false);
		ManagedUsersDatabase.addAssignedUser(newUser);
		HumanListener.currentInt.showAssignMovedDone(user.getNick());
		return;
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#deleteDownloadedMsgs(common.User, java.lang.String)
	 */
	public void deleteDownloadedMsgs(User user, String pw) throws UserNotManagedException, WrongPasswordException, UserBlockedException{
		ManagedUser man;
		synchronized(ManagedUsersDatabase.class){
			HumanListener.currentInt.showDeleteDownloadedRequest(user.getNick());
			if ((man = ManagedUsersDatabase.getManaged(user))==null) {
				HumanListener.currentInt.showWrongUser(user.getNick());
				throw new UserNotManagedException();
			}
			if (!man.checkPassword(pw)) {
				HumanListener.currentInt.showWrongUserPassword(user.getNick());
				throw new WrongPasswordException();
			}
			if (man.isBlocked()) throw new UserBlockedException();
			
			HumanListener.currentInt.showDeleteDownloadedDone(user.getNick());
			man.deleteDownloadedQueue();
			return;
		}
	}

	/**
	 * Sends to the bootstrap the current load via a UDP packet.
	 */
	public static void sendLoad() {
		Long freeMem = Runtime.getRuntime().freeMemory();
		String dataString = ""+freeMem.toString()+" "+Integer.toString(Statistics.get24HourFwMsgs()).toString();
		HumanListener.currentInt.showSendLoad(dataString);
		DatagramPacket pkt = new DatagramPacket(dataString.getBytes(), dataString.length());
		pkt.setPort(Cache.getBS().getUDPPort());
		pkt.setAddress(Cache.getBS().getIP());
		try{
			DatagramSocket sock = new DatagramSocket(MyselfMessageServer.myself.getPort());
			sock.send(pkt);
			sock.close();
		}catch (SocketException e){
			HumanListener.currentInt.showUDPSocketProblem();
		}catch (IOException e){
			HumanListener.currentInt.showSendLoadTransferProblem();
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#blockContact(common.User, java.lang.String, common.User)
	 */
	public void blockContact(User user, String pw, User userToBlock) throws UserNotManagedException, UserNotExistsException, WrongPasswordException, UserBlockedException{
		HumanListener.currentInt.showBlockContactRequest(user.getNick(),userToBlock.getNick());
		synchronized(ManagedUsersDatabase.class){
			ManagedUser man = ManagedUsersDatabase.getManaged(user);
			if (man==null) throw new UserNotManagedException();
			if (!man.checkPassword(pw)) throw new WrongPasswordException();
			if (man.isBlocked()) throw new UserBlockedException();
			synchronized (man) {
					man.blockFriend(userToBlock);
					HumanListener.currentInt.showBlockContactDone(user.getNick(),userToBlock.getNick());
			}
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#getUserContact(common.User)
	 */
	public User getUserContact(User user) throws RemoteException, UserNotExistsException, GenericErrorException, UserBlockedException {
		HumanListener.currentInt.showGetContactRequest(user.getNick());
		synchronized(ManagedUsersDatabase.class){
			ManagedUser man = ManagedUsersDatabase.getManaged(user);
			if (man==null){
				HumanListener.currentInt.showGetContactCache(user.getNick());
				try{
					return Cache.getRemoteContact(user);
				} catch (SystemErrorException e){
					HumanListener.currentInt.showSystemError(user);
					throw new GenericErrorException();
				}
			}
			else if (man.isBlocked()) throw new UserBlockedException();
			else{
				HumanListener.currentInt.showGetContactDone(user.getNick());
				return man.copyContactTo(user);
			}
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForServletInterface#removeContact(common.User, java.lang.String, common.User)
	 */
	public void removeContact(User user, String pw, User userToRemove) throws UserNotManagedException, WrongPasswordException, UserBlockedException {
		HumanListener.currentInt.showRemoveContactRequest(user.getNick(),userToRemove.getNick());
		synchronized(ManagedUsersDatabase.class){
			ManagedUser man = ManagedUsersDatabase.getManaged(user);
			if (man==null) throw new UserNotManagedException();
			if (!man.checkPassword(pw)) throw new WrongPasswordException();
			if (man.isBlocked()) throw new UserBlockedException();
			
			synchronized (man) {
					man.removeContact(user);
					HumanListener.currentInt.showRemoveContactDone(user.getNick(),userToRemove.getNick());
			}
		}
	}

	/* (non-Javadoc)
	 * @see common.interfaces.MSForMSInterface#getManagedUserContact(common.User)
	 */
	public User getManagedUserContact(User user) throws RemoteException, UserNotManagedException {
		HumanListener.currentInt.showGetManagedContactRequest(user.getNick());
		synchronized(ManagedUsersDatabase.class){
			ManagedUser man = ManagedUsersDatabase.getManaged(user);
			if (man==null) {
				HumanListener.currentInt.showGetManagedContactUserNotManaged(user.getNick());
				throw new UserNotManagedException();
			}
			else {
				HumanListener.currentInt.showGetManagedContactDone(user.getNick());
				return man.copyContactTo(user);
			}
				
		}
	}
}
