package messageServerP;

import java.io.*;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


public class FileBackupper implements BackupperInterface {

	private File bkFile=null;
	
	
	/**Creates a backupper that writes on a single file.
	 * @param backupFilePath the backup file path.
	 */
	public FileBackupper(String backupFilePath){
		bkFile = new File(backupFilePath);
	}
	/**
	 * @return the name of the file this backup is using.
	 */
	public String getFilename(){
		return  bkFile.getAbsolutePath();
	}
	
	/* (non-Javadoc)
	 * @see messageServerP.BackupperInterface#backupWholeData(java.util.List)
	 */
	public synchronized void backupWholeData(List<ManagedUser> users) throws IOException{
		bkFile.createNewFile();
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(bkFile));
		
		Iterator<ManagedUser> usersIt = users.iterator();
		while (usersIt.hasNext()){
			oos.writeObject(usersIt.next());
		}
		oos.flush();
		oos.close();
	}

	/**Adds or removes a single managed user from the backup.
	 * @param usr the user to add or remove (in the latter case, only the nickname is considered).
	 * @param remove true if the user must be removed, false otherwise.
	 * @throws IOException if the file could not be modified or read.
	 */
	private synchronized void backupSingleData(ManagedUser usr, boolean remove) throws IOException{
		List<ManagedUser> users = new Vector<ManagedUser>();
		try{
			restoreData(users);
		} catch (Exception e){
			throw new IOException();
		}
		if (remove)
			users.remove(usr);
		else
			users.add(usr);
		
		backupWholeData(users);
	}
	
	/* (non-Javadoc)
	 * @see messageServerP.BackupperInterface#backupUser(messageServerP.ManagedUser)
	 */
	public synchronized void backupUser(ManagedUser usr) throws IOException {
		backupSingleData(usr, false);
	}

	/* (non-Javadoc)
	 * @see messageServerP.BackupperInterface#removeBackuppedUser(messageServerP.ManagedUser)
	 */
	public synchronized void removeBackuppedUser(ManagedUser usr) throws IOException{
		backupSingleData(usr, true);
	}

	/* (non-Javadoc)
	 * @see messageServerP.BackupperInterface#restoreData(java.util.List)
	 */
	public synchronized int restoreData(List<ManagedUser> restoredUsers) throws IOException, ClassNotFoundException, IllegalArgumentException{
		bkFile.createNewFile();
		if (restoredUsers==null || !restoredUsers.isEmpty()) throw new IllegalArgumentException();
		
		int count=0;
		FileInputStream fis = new FileInputStream(bkFile);
		ObjectInputStream ois;
		try{
			ois = new ObjectInputStream(fis);
		} catch (EOFException e){
			return 0;
		}
		while (fis.available()>0){
			Object readObj = ois.readObject();
			count++;
			if (readObj instanceof ManagedUser)  restoredUsers.add((ManagedUser) readObj);
			else throw new ClassCastException();
		}
		return count;
	}
	/* (non-Javadoc)
	 * @see messageServerP.BackupperInterface#updateUser(messageServerP.ManagedUser)
	 */
	public synchronized void updateUser(ManagedUser newUserStatus) throws IOException {
		List<ManagedUser> users = new Vector<ManagedUser>();
		try{
			restoreData(users);
		} catch (Exception e){
			throw new IOException();
		}
		users.remove(newUserStatus);
		users.add(newUserStatus);
		
		backupWholeData(users);
	}
}
