//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : ManagedUser.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//

package messageServerP;


import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;
import common.*;
import common.interfaces.exceptions.UserAlreadyInsertedException;

/**A representation of a user managed by the Message Server.
 * Any affection of a user data is backupped on the main backupper.
 * @author Marco Cornolti
 */
public class ManagedUser extends common.User implements Cloneable {
	private static final long serialVersionUID = 1L;
	private String password;
	private boolean blocked;
    private Vector<Message> msgsQueue;
    private int downloadedMessages;
    private ContactList list;
    
    /**Creates a Managed User.
     * @param password the managed user password.
     * @param nick the user nickname.
     * @param ip the user ip address.
     * @param port the user port.
     */
    public ManagedUser(String nick, InetAddress ip, int port, String password){
        super(nick, ip, port, false, false);
        this.password=password;
        msgsQueue=new Vector<Message>();
        this.blocked=false;
        this.list=new ContactList();
    }
    
	/**Checks the password
	 * @param pw the password to check
	 * @return true if and only if the password given by argument corresponds to the user password.
	 */
	public synchronized boolean checkPassword(String pw) {
	    return pw.equals(password);
	}
	
	/**Adds a message to the incoming queue of the user
	 * @param ms the message to add
	 */
	public synchronized void addMsgToQueue(common.Message ms) {
		synchronized (msgsQueue){
	    	msgsQueue.add(ms);}
		refreshBackup();
	}
	
	/**Returns the messagges in the queue and marks it as downloaded, ready to be deleted.
	 * @return the queued messages.
	 */
    public synchronized Vector<Message> getQueue() {
    	synchronized (msgsQueue){
    		downloadedMessages = msgsQueue.size();
    		return msgsQueue;
        }
    }
    /**Deletes the already downloaded messages.
     */
    public synchronized void deleteDownloadedQueue() {
    	synchronized (msgsQueue){
    		for (int i=0;i<downloadedMessages;i++)
    			msgsQueue.remove(0);
    		downloadedMessages=0;
    	}
    	refreshBackup();
    }
	
	/**Check if a user can perform any action. 
	 * @return true if the user il blocked
	 */
	public synchronized boolean isBlocked() {
	    return this.blocked;
	}
    
    /**Blocks the user to not let it perform any action.
     * @param blocked true if the user must be set to blocked, false otherwise.
     */
    public synchronized void block(boolean blocked){
        this.blocked=blocked;
        refreshBackup();
    }
	
	/**Adds a friend to the incoming contact list of this user.
	 * @param user the user to add
	 */
    public synchronized void addFriend(common.User user) throws UserAlreadyInsertedException{
	    this.list.addUser(user,true);
	    refreshBackup();
    }

	
    /**Sets a contact as blocked (no more messages by this user are accepted).
     * @param user the user to block
     */
    public synchronized void blockFriend(common.User user) {
	    this.list.setFriend(user, false);
	    refreshBackup();
	}
	
    /**Removes a user from this user contact list.
     * @param user the user to remove.
     */
    public synchronized void removeContact(common.User user) {
	    this.list.removeUser(user);
	    refreshBackup();
	}
	
	/**Checks if a user given by argument accepts messages from this managed user.
	 * @param user the user.
	 * @return true if and only if the Managed User accepts messages from the User given by argument
	 */
	public synchronized boolean isFriendOf(common.User user) {
	    return this.list.accepts(user);
	}
	
    /**Changes this user password.
     * @param oldPw the old password.
     * @param newPw the new password.
     * @return true only if the old password is right and the new password has been set.
     */
    public synchronized boolean changePassword(String oldPw,String newPw) {
        if (this.checkPassword(oldPw)) {
        	this.password=newPw;
            refreshBackup();
        }
        else return false;
        return true;
    }
    
    /**Gets the user's Contact List.
     * @return the user's Contact List
     */
    public synchronized ContactList getContactList(){
        return this.list;
    }
    
    /** Copies IP and port of this user to the user passed by argument. 
     * @param user the user to copy the IP and port to.
     * @return the user passed by argument with the new cloned IP and port.
     */
    public User copyContactTo(User user){
		User manCopy = this.clone();
		user.setIp(manCopy.getIP());
		user.setPort(manCopy.getPort());
		return user;	
    }
    
    /**
     * Refreshes the main backup updating it with the state of this user.
     */
    public synchronized void refreshBackup(){
    	try{
			MessageServerCore.mainBackupper.updateUser(this); 	
		} catch (Exception e){
			HumanListener.currentInt.showCouldNotBackupUser(this.getNick());
		}
    }
    
    /**Tests if a user given as argument has the same nickname of this one.
     * @param us2 the user whose nickname must be checked.
     * @return true if the two users nicknames are equal.
     */
    public boolean equals(ManagedUser us2){
    	return us2.getNick().equals(this.getNick());
    }
    
    public synchronized ManagedUser clone(){
    	ManagedUser ret=null;
    	try{
    		InetAddress clonedIP;
    		if (this.getIP()!=null) clonedIP = InetAddress.getByName(this.getIP().getHostAddress());
    		else clonedIP=null;
    		
    	    ret = new ManagedUser(new String (this.getNick()), clonedIP,this.getPort(),new String(this.password));
    	    ret.downloadedMessages=this.downloadedMessages;
    	    ret.blocked=this.blocked;
    	    ret.list=this.list.clone();
    	    ret.msgsQueue = new Vector<Message>();
    	    for (int i=0;i<this.msgsQueue.size();i++)
    	    	ret.msgsQueue.add(msgsQueue.get(i).clone());
    	    
    	}catch (UnknownHostException e){
    		return null;
    	}
    	return ret;
    	
    }
}
