//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : ManagedUsersDatabase.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//



package messageServerP;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import common.User;


/**The database of managed users.
 * @author Marco Cornolti
 */
public class ManagedUsersDatabase {
	private static boolean assignBlocked=false;
    private static List<ManagedUser> users =new Vector<ManagedUser>();
	/**Takes log(N) where N is the number of administrated users.
	 * @param user the user to search for (only the nickname is considered)
	 * @return the user if it is managed by the local Message Server, <b>null</b> otherwise
	 */
	public static synchronized ManagedUser getManaged(common.User user) {
		int idx=-1;
		idx=Collections.binarySearch(users, user);
		if (idx<0) return null;
	    return users.get(idx);
	}
    /**Checks if a user is friend of another.
     * Takes log(N) where N is the number of administrated users.
     * @param u1 the user.
     * @param u2 the friend.
     * @return true if u1 is friend of u2.
     */
    public static synchronized boolean userIsFriendOf(common.User u1, common.User u2) {
        ManagedUser man = getManaged(u1);
        if (man==null) return false;
        if (man.isFriendOf(u2)) return true;
        return false;
     }
    
    /**Deletes a user. 
     * Takes log(N) where N is the number of administrated users.
     * @param userToDelete the user to delete (only the nickname is considered).
     * @return true if the user was present and has been removed.
     */
    public static synchronized boolean deleteUser(User userToDelete){
    		int idx = Collections.binarySearch(users, userToDelete);
        	if (idx<0) return false;
        	ManagedUser manToDel = users.get(idx);
        	try{
        		MessageServerCore.mainBackupper.removeBackuppedUser(manToDel);
        	} catch (Exception e){
        		HumanListener.currentInt.showCouldNotBackupUser(userToDelete.getNick());
        	}
            users.remove(idx);
            return true;
    }
    /**Sets the block status of the Message Server's Managed Users.
     * When the server is blocked, no users can be assigned nor deleted.
     * They can only be moved to another server.
     * @param status the status to set
     */
    public static synchronized void blockAssignation(boolean status){
        assignBlocked=status;
    }
    /**
     * @return true if the assignations to this server are blocked.
     */
    public static synchronized boolean assignationBlocked(){
        return assignBlocked;
    }
    /** Blocks a user to not let it receive more messages.
     * @param user the user to block
     */
    public static synchronized void blockUser(User user, boolean blocked){
        	ManagedUser man = getManaged(user);
            if (man==null) return;
            man.block(blocked);
    }
    
    /**Creates a user and adds it to the list.
     * Takes O(n log(n)) where n is the number of users contained in the list.
     * @param user the user to create.
     * @param password the user's password.
     */
    public static synchronized void createUser(User user, String password) {
    	ManagedUser newUsr = new ManagedUser(user.getNick(),user.getIP(),user.getPort(),password);
    	users.add(newUsr);
    	Collections.sort(users);
    	try{
    		MessageServerCore.mainBackupper.backupUser(newUsr); 	
    	} catch (Exception e){
    		HumanListener.currentInt.showCouldNotBackupUser(user.getNick());   
    	}
    }
    /**Adds an already existing user to the managed user list.
     * Takes O(n log(n)) where n is the number of users contained in the list.
     * @param user the managed user to add
     */
    public static synchronized void addAssignedUser(ManagedUser user){
      		users.add(user);
        	Collections.sort(users);
      		try{
      			MessageServerCore.mainBackupper.updateUser(user); 	
      		} catch (Exception e){
      			HumanListener.currentInt.showCouldNotBackupUser(user.getNick());
      	}
    }
    
    /**
     * @return the number of managed users.
     */
    public static int getManagedUsers(){
        return users.size();
    }
    
    /**Copies the managed users into an array and returns the array.
     * Takes O(N) where N is the number of managed users.
     * The method is synchronized but the Users datas must not be modified.
     * @return The managed users.
     */
    public static synchronized ManagedUser[] getManagedUsersArray(){
    	    ManagedUser[] usersArray = new ManagedUser[users.size()];
            Iterator<ManagedUser> it = users.iterator();
            int i=0;
            while (it.hasNext())
            	usersArray[i++]=it.next();
            return usersArray;
    	
    }
    
    /**Takes O(N) where N is the number of managed users.
     * @return the number of online managed users.
     */
    public static synchronized int getOnlineManagedUsers(){
        int online=0;
        for (int i=0;i<users.size();i++)
        	if (users.get(i).isOnline()) online++;			
        return online;
    }
    
    /**Takes O(N) where N is the number of managed users.
     * @return the number of queued messages.
     */
    public static synchronized int getQueuedMessagesNum(){
        int msgs=0;

        	for (int i=0;i<users.size();i++)
        		msgs += users.get(i).getQueue().size();
        return msgs;
    }
    
    /**
     * @return true if and only if there are no users assigned to this message server. 
     */
    public static synchronized boolean noneAssigned(){
        	return users.size()==0;			
    }
    
	/**Restores the managed users list replacing the current one with the one given by argument. Must only be used at boot time.
	 * @param restoredUsers the new users list.
	 */
	public static synchronized void restoreData(List<ManagedUser> restoredUsers) {
		users=restoredUsers;
	}
	/**
	 * @return the managed users list. Can be used only for reading (e.g. backupping).
	 */
	public static synchronized List<ManagedUser> getManagedUsersList() {
		return users;
	}
}
