//
//
//  Generated by StarUML(tm) Java Add-In
//
//  @ Project : Untitled
//  @ File Name : MessageServerCore.java
//  @ Date : 19/03/2007
//  @ Author : 
//
//



package messageServerP;

import java.io.IOException;
import java.io.InvalidClassException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.*;
import java.util.List;
import java.util.Vector;

import common.Bootstrap;

/** The main class of the Message Server, witch starts all the services.
 * @author Marco Cornolti
 *
 */
public class MessageServerCore {
	static protected BackupperInterface mainBackupper;

	/**The main method to launch the Message Server, activating all the threads, publishing the rmi services, and restoring the data from the filename previousely used as backup.
	 * @param args the arguments.
	 */
	public static void main(String[] args) {
        
        
        int rmiPort=-1;
        int bsRmiPort=-1;
        int bsUdpPort=-1;
        String backupFile=null;
        
        if (args.length<5){
            System.out.println("Gossip launching syntax:\n gossip rmi_port bootstrap_hostname bootstrap_RMIport bootstrap_UDPport ms_admin_password [users_file]");
            System.exit(-1);
        }
        try{
            rmiPort=Integer.parseInt(args[0]);
            bsRmiPort=Integer.parseInt(args[2]);
            bsUdpPort=Integer.parseInt(args[3]);
        } catch (NumberFormatException e){
            System.out.println("Could not parse a port number. Strings ["+args[0]+"] and ["+args[2]+"] are not valid.");
            System.exit(-1);
        }
        if (args.length>=6) backupFile=args[5];
        
        System.out.println("Loading GOSSIP Message Server...");
        
        if (backupFile==null)
        	System.out.println("WARNING: the Massage Server is NOT going to save permanently any data, so every time it gets restarted all the users data will be lost. To save the data, restart the server specifying a file to save the data to.");
        else{
        	try {
				System.out.println("Using file "+backupFile+" as backup.");
                mainBackupper = new FileBackupper(backupFile);
                List<ManagedUser> restoredUsers = new Vector<ManagedUser>();
                int restoredDataN = mainBackupper.restoreData(restoredUsers);
                ManagedUsersDatabase.restoreData(restoredUsers);
			    if ( restoredDataN != 0)
                	System.out.println("Restoring of "+restoredDataN+" backupped records done.");
                else {
                	System.out.println("Backup file is empty and will be used to store new data.");
                }
        	} catch (InvalidClassException e) {
        		System.out.println("Could not read data from input file. Saved datas version and current Bootstrap version may differ.");
        		System.exit(-1);
        	} catch (StreamCorruptedException e) {
        		System.out.println("Could not read data from input file. The input stream is corrupted.");
        		System.exit(-1);
        	} catch (OptionalDataException e) {
           		System.out.println("Could not read data from input file. Unexpected data in file.");
        		System.exit(-1);
 			} catch (IllegalArgumentException e) {
 				System.out.println("Could not read data from input file. Internal error.");
        		System.exit(-1);
        	} catch (IOException e) {
        		System.out.println("Could not read data from input file. I/O error.");
        		System.exit(-1);
        	} catch (ClassNotFoundException e) {
        		System.out.println("Could not read data from input file. Saved datas version and current Bootstrap version may differ.");
        		System.exit(-1);
        	}
        }
        
       
        
        try{
        	MyselfMessageServer.makeMyselfMessageServer(InetAddress.getLocalHost(), rmiPort, args[4]);
        } catch (UnknownHostException e){
        	System.out.println("Unable to obtain local host ip address.");
        	System.exit(-1);
        }
        
        String bsHost=args[1];
        System.out.print("Connecting to GOSSIP Bootstrap RMI service at "+bsHost+":"+bsRmiPort+"... ");
        
        try{
            Cache.setBootstrap(new Bootstrap(bsHost,bsRmiPort,bsUdpPort));
            System.out.println("Done.");
        } catch (RemoteException e){
            System.out.println("Failed. Could not contact remote registry. This may happen if the Bootstrap server is down or if the Message Server version is not the same of the Bootstrap.");
            System.exit(-1);
        } catch (NotBoundException e){
            System.out.println("Failed. The GOSSIP Bootstrap service is not bound to the registry.");
            System.exit(-1);
        } catch (Exception e){
            System.out.println("Failed with exception: "+e);
        }
        
        try{
            System.out.print("Creating local registry at localhost:"+rmiPort+"... ");
            Registry MSreg = LocateRegistry.createRegistry(rmiPort);
            MSreg.bind("gossipMS", new Communication());
            System.out.println("Done.");
        } catch (RemoteException e){
            System.out.println("Failed. Could not create RMI Registry. Maybe the port is already in use?");
            System.exit(-1);
        } catch (AlreadyBoundException e){
            System.out.println("Failed. The service is already started.");
            System.exit(-1);
        }
        
        System.out.print("Starting load sender thread on UDP port "+rmiPort+"... ");
        Thread loadThread = new LoadThread();
        loadThread.start();
        System.out.println("Done.");
        
        System.out.print("Starting human interface... ");
        Thread humThread = new HumanListener();
        humThread.start();
        System.out.println("Done.");
        
	}
}
