package servlet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import common.ContactList;
import common.Message;
import common.User;


/**
 * This graphic interface shows main window of the system.
 * There are two constructors. The first shows the access window.
 * The second shows the status window.
 * @author Alessandro Franchini
 *
 */

public class GossipInterface extends JFrame {
	
	public static Vector <Connection> conn;
	ClientListener tt = null;
	private static final long serialVersionUID = 1L;
	public static ContactList utenti;
	public static DefaultListModel inLinea,offLinea,neri;
	public static LoginInterface li;
	public static JList onLine, offLine;
	private JPanel pannello;
	private JMenuBar menu = new JMenuBar();
	private JMenu file = new JMenu("File");
	private JMenu infoCont = new JMenu("Contatti");
	//private JMenu help = new JMenu("Help");
	private JMenuItem contatti = new JMenuItem("Gestione contatti");
	private JMenuItem logout = new JMenuItem("Logout");
	private JMenuItem esci = new JMenuItem("Esci");
	private JButton avvia;
	
	/**
	 * The first constructor. Initialises the window access.
	 *
	 */
	public GossipInterface(){
		setTitle("GOSSIP");
		setBackground(Color.BLUE);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pannello = new JPanel();
		pannello.setLayout(new BorderLayout());
		ImageIcon logo = new ImageIcon("gossip_logo.jpg");
		JLabel avatar = new JLabel(logo);
		avatar.setBounds(45,50,200,50);
		avatar.setSize(200,200);
		pannello.add(avatar);
		JButton login = new JButton("Accedi");
		getRootPane().setDefaultButton(login);
		login.setEnabled(true);
		login.setBounds(80,300,120,30);
		pannello.add(login,BorderLayout.SOUTH);
		login.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				boolean flag = AccessOperation.setConnBoot(ClientCore.server.getIP(),ClientCore.server.getRMIPort());
				if (flag){
					li = new LoginInterface();
					li.setVisible(true);
				}
			}
			
		});
		getContentPane().add(pannello);
	}
	
	/**
	 * The second constructor. Shows the main window.
	 * @param utente the user
	 * @param list the contact list
	 * @param message the received messages
	 */
	public GossipInterface(User utente,ContactList list, Vector <Message> message){
		conn = new Vector<Connection>();
		utenti = list;
		utenti = setStatus(utenti);
		setModello(utenti);
		setTitle("GOSSIP");
		setSize(300,500);
		setBackground(Color.BLUE);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		pannello = new JPanel();
		pannello.setLayout(null);
		JLabel nick = new JLabel(utente.getNick()+ " (In linea)");
		nick.setBounds(100,10,100+(utente.getNick().length()*5),20);
		pannello.add(nick);
		JLabel statoOn = new JLabel("Utenti online");
		statoOn.setBounds(10,40,100,20);
		pannello.add(statoOn);
		onLine = new JList(inLinea);
		onLine.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		onLine.setSelectedIndex(0);
		JScrollPane scrollOn = new JScrollPane(onLine);
		scrollOn.setBounds(10,60,270,150);
		pannello.add(scrollOn);
		JLabel statoOff = new JLabel("Utenti offline");
		statoOff.setBounds(10,230,100,20);
		pannello.add(statoOff);
		offLine = new JList(offLinea);
		offLine.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane scrollOff = new JScrollPane(offLine);
		scrollOff.setBounds(10,250,270,150);
		pannello.add(scrollOff);
		avvia = new JButton("Avvia conversazione");
		avvia.setBounds(10,410,270,25);
		pannello.add(avvia);
		getRootPane().setDefaultButton(avvia);
		file.add(logout);
		file.add(esci);
		menu.add(file);
		infoCont.add(contatti);
		menu.add(infoCont);
		//menu.add(help);
		setJMenuBar(menu);
		setupEventi();
		getContentPane().add(pannello);
		tt = new ClientListener();
		tt.start();
		this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	logOff();
            	System.exit(0); 
            	}
        });
	}
	
	/**
	 * This method manages the events joined to the graphic interface.
	 *
	 */
	void setupEventi(){
		onLine.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(e.getClickCount()==1){
					offLine.clearSelection();
				}	
			}
		});
		offLine.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(e.getClickCount()==1){
					onLine.clearSelection();
				}	
			}
		});
		avvia.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				String temp;
				if((temp = (String) onLine.getSelectedValue())!=null){
					User tmp = utenti.getUser(temp);
					if(tmp.getConn()==1){
						return;
					}
					else {
						
						PeerCommunication tt = new PeerCommunication(null,tmp,null);
						tt.start();
					}
				}
				if((temp = (String) offLine.getSelectedValue())!=null){
					User tmp = utenti.getUser(temp);
					if(tmp.getConn()==1){
						return;
					}
					else {
						PeerCommunication tt = new PeerCommunication(null,tmp,null);
						tt.start();
					}
				}
			}
		});
		logout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				logOff();
				try {
					tt.ss.close();
				} catch (IOException e1) {
					System.out.println("A problem occurs");
				}
				ClientCore.gi.dispose();
				ClientCore.gi = new GossipInterface();
				ClientCore.gi.pack();
				ClientCore.gi.setVisible(true);
			}
		});
		esci.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				logOff();
				System.exit(1);
			}
			
		});
		contatti.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				ContattInterface ci = new ContattInterface();
				ci.setVisible(true);
				
			}
			
		});
	}
	
	
	/**
	 * Differentiates the user according to their state.
	 * @param modello the contacts list
	 */
	void setModello(ContactList modello){
		inLinea = new DefaultListModel();
		offLinea = new DefaultListModel();
		neri = new DefaultListModel();
		for(int i = 0; i<modello.getSize();i++){
			User temp = (User) modello.getUser(i);
			if(temp.isFriend()){
				if(temp.isOnline()){
					inLinea.addElement(temp.getNick());
				}
				else
					offLinea.addElement(temp.getNick());
			}
			else
				neri.addElement(temp.getNick());
		}
	}
	
	/**
	 * Checks which users are on line.
	 * @param model the contacts list.
	 * @return the new contacts list.
	 */
	ContactList setStatus(ContactList model){
		ControlStatus cs = new ControlStatus(model,ClientCore.myself);
		int size = model.getSize();
		for(int i = 0; i<size;i++){
			threadStatus ts = new threadStatus(cs);
			ts.start();
		}
		return cs.getFinalList();
	}
	
	/**
	 * Effects the logout operations.
	 *
	 */
	void logOff(){
		for (int i=0; i<utenti.getSize(); i++){
			User temp = null;
			if((temp = utenti.getUser(i)).isOnline() && temp.isFriend()){
				Socket so = null;
				Message msg = new Message("Type::Offline",temp,ClientCore.myself);
				OutputStream os;
				try {
					so = new Socket(temp.getIP(),temp.getPort());
					os = so.getOutputStream();
					ObjectOutputStream oos = new ObjectOutputStream(os);
					oos.writeObject(msg);
					} catch (Exception e) {
						System.out.println("The user is offline");
						
				}
			}
		}
		for(int i = 0; i<conn.size(); i++){
			conn.get(i).closeConnection();
		}
		AccessOperation.logOff();
	}
	
	/**
	 * This method change the location of user in graphic interface
	 * @param user the user to change
	 * @param form selected the list
	 * @param which 0 to add; 1 to remove.
	 */
	static void repaintWindow(User user, int form, int which){
		if(form==0){
			if(which==0){
				GossipInterface.offLinea.addElement(user.getNick());
				GossipInterface.offLine.repaint();
			}
			else{
				GossipInterface.offLinea.removeElement(user.getNick());
				GossipInterface.offLine.repaint();
			}
		}
		if(form==1){
			if(which==0){
				GossipInterface.inLinea.addElement(user.getNick());
				GossipInterface.onLine.repaint();
			}
			else{
				GossipInterface.inLinea.removeElement(user.getNick());
				GossipInterface.onLine.repaint();
			}
			
		}	
		
	}
	
	

}
